@@ -351,13 +351,26 @@ class EccKey(object):
 
         return EccKey(curve="P-256", point=self.pointQ)
 
-    def _export_subjectPublicKeyInfo(self):
+    def _export_subjectPublicKeyInfo(self, compress):
+    
+        # See 2.2 in RFC5480 and 2.3.3 in SEC1
+        # The first byte is:
+        # - 0x02:   compressed, only X-coordinate, Y-coordinate is even
+        # - 0x03:   compressed, only X-coordinate, Y-coordinate is odd
+        # - 0x04:   uncompressed, X-coordinate is followed by Y-coordinate
+        #
+        # PAI is in theory encoded as 0x00.
 
-        # Uncompressed form
         order_bytes = _curve.order.size_in_bytes()
-        public_key = (bchr(4) +
-                      self.pointQ.x.to_bytes(order_bytes) +
-                      self.pointQ.y.to_bytes(order_bytes))
+
+        if compress:
+            first_byte = 2 + self.pointQ.y.is_odd()
+            public_key = (bchr(first_byte) +
+                          self.pointQ.x.to_bytes(order_bytes))
+        else:
+            public_key = (bchr(4) +
+                          self.pointQ.x.to_bytes(order_bytes) +
+                          self.pointQ.y.to_bytes(order_bytes))
 
         unrestricted_oid = "1.2.840.10045.2.1"
         return _create_subject_public_key_info(unrestricted_oid,
@@ -402,8 +415,8 @@ class EccKey(object):
                             **kwargs)
         return result
 
-    def _export_public_pem(self):
-        encoded_der = self._export_subjectPublicKeyInfo()
+    def _export_public_pem(self, compress):
+        encoded_der = self._export_subjectPublicKeyInfo(compress)
         return PEM.encode(encoded_der, "PUBLIC KEY")
 
     def _export_private_pem(self, passphrase, **kwargs):
@@ -421,16 +434,21 @@ class EccKey(object):
         encoded_der = self._export_pkcs8(passphrase=passphrase, **kwargs)
         return PEM.encode(encoded_der, "ENCRYPTED PRIVATE KEY")
 
-    def _export_openssh(self):
-        assert not self.has_private()
+    def _export_openssh(self, compress):
+        if self.has_private():
+            raise ValueError("Cannot export OpenSSH private keys")
 
         desc = "ecdsa-sha2-nistp256"
-
-        # Uncompressed form
         order_bytes = _curve.order.size_in_bytes()
-        public_key = (bchr(4) +
-                      self.pointQ.x.to_bytes(order_bytes) +
-                      self.pointQ.y.to_bytes(order_bytes))
+        
+        if compress:
+            first_byte = 2 + self.pointQ.y.is_odd()
+            public_key = (bchr(first_byte) +
+                          self.pointQ.x.to_bytes(order_bytes))
+        else:
+            public_key = (bchr(4) +
+                          self.pointQ.x.to_bytes(order_bytes) +
+                          self.pointQ.y.to_bytes(order_bytes))
 
         comps = (tobytes(desc), b("nistp256"), public_key)
         blob = b("").join([ struct.pack(">I", len(x)) + x for x in comps])
@@ -441,9 +459,14 @@ class EccKey(object):
 
         Args:
           format (string):
-            The format to use for wrapping the key:
+            The format to use for encoding the key:
 
-            - *'DER'*. The key will be encoded in an ASN.1 DER_ structure (binary).
+            - *'DER'*. The key will be encoded in ASN.1 DER format (binary).
+              For a public key, the ASN.1 ``subjectPublicKeyInfo`` structure
+              defined in `RFC5480`_ will be used.
+              For a private key, the ASN.1 ``ECPrivateKey`` structure defined
+              in `RFC5915`_ is used instead (possibly within a PKCS#8 envelope,
+              see the ``use_pkcs8`` flag below).
             - *'PEM'*. The key will be encoded in a PEM_ envelope (ASCII).
             - *'OpenSSH'*. The key will be encoded in the OpenSSH_ format
               (ASCII, public keys only).
@@ -455,7 +478,7 @@ class EccKey(object):
             If ``True`` (default and recommended), the `PKCS#8`_ representation
             will be used.
 
-            If ``False``, the much weaker and `PEM encryption`_ mechanism will be used.
+            If ``False``, the much weaker `PEM encryption`_ mechanism will be used.
 
           protection (string):
             When a private key is exported with password-protection
@@ -463,6 +486,13 @@ class EccKey(object):
             present and be a valid algorithm supported by :mod:`Cryptodome.IO.PKCS8`.
             It is recommended to use ``PBKDF2WithHMAC-SHA1AndAES128-CBC``.
 
+          compress (boolean):
+            If ``True``, a more compact representation of the public key
+            (X-coordinate only) is used.
+
+            If ``False`` (default), the full public key (in both its
+            coordinates) will be exported.
+
         .. warning::
             If you don't provide a passphrase, the private key will be
             exported in the clear!
@@ -472,11 +502,12 @@ class EccKey(object):
             (both ``DER`` and ``PEM`` formats), any extra parameters
             is passed to :mod:`Cryptodome.IO.PKCS8`.
 
-        .. _DER:        http://www.ietf.org/rfc/rfc5915.txt
         .. _PEM:        http://www.ietf.org/rfc/rfc1421.txt
         .. _`PEM encryption`: http://www.ietf.org/rfc/rfc1423.txt
         .. _`PKCS#8`:   http://www.ietf.org/rfc/rfc5208.txt
         .. _OpenSSH:    http://www.openssh.com/txt/rfc5656.txt
+        .. _RFC5480:    https://tools.ietf.org/html/rfc5480
+        .. _RFC5915:    http://www.ietf.org/rfc/rfc5915.txt
 
         Returns:
             A multi-line string (for PEM and OpenSSH) or bytes (for DER) with the encoded key.
@@ -486,6 +517,8 @@ class EccKey(object):
         ext_format = args.pop("format")
         if ext_format not in ("PEM", "DER", "OpenSSH"):
             raise ValueError("Unknown format '%s'" % ext_format)
+        
+        compress = args.pop("compress", False)
 
         if self.has_private():
             passphrase = args.pop("passphrase", None)
@@ -516,11 +549,11 @@ class EccKey(object):
             if args:
                 raise ValueError("Unexpected parameters: '%s'" % args)
             if ext_format == "PEM":
-                return self._export_public_pem()
+                return self._export_public_pem(compress)
             elif ext_format == "DER":
-                return self._export_subjectPublicKeyInfo()
+                return self._export_subjectPublicKeyInfo(compress)
             else:
-                return self._export_openssh()
+                return self._export_openssh(compress)
 
 
 def generate(**kwargs):
@@ -601,44 +634,87 @@ def construct(**kwargs):
     return EccKey(**kwargs)
 
 
-def _import_public_der(curve_name, publickey):
+def _import_public_der(curve_oid, ec_point):
+    """Convert an encoded EC point into an EccKey object
+
+    curve_name: string with the OID of the curve
+    ec_point: byte string with the EC point (not DER encoded)
+
+    """
 
     # We only support P-256 named curves for now
-    if curve_name != _curve.oid:
+    if curve_oid != _curve.oid:
         raise UnsupportedEccFeature("Unsupported ECC curve (OID: %s)" % curve_name)
 
-    # ECPoint ::= OCTET STRING
+    # See 2.2 in RFC5480 and 2.3.3 in SEC1
+    # The first byte is:
+    # - 0x02:   compressed, only X-coordinate, Y-coordinate is even
+    # - 0x03:   compressed, only X-coordinate, Y-coordinate is odd
+    # - 0x04:   uncompressed, X-coordinate is followed by Y-coordinate
+    #
+    # PAI is in theory encoded as 0x00.
 
-    # We support only uncompressed points
     order_bytes = _curve.order.size_in_bytes()
-    if len(publickey) != (1 + 2 * order_bytes) or bord(publickey[0]) != 4:
-        raise UnsupportedEccFeature("Only uncompressed points are supported")
+    point_type = bord(ec_point[0])
+    
+    # Uncompressed point
+    if point_type == 0x04:
+        if len(ec_point) != (1 + 2 * order_bytes):
+            raise ValueError("Incorrect EC point length")
+        x = Integer.from_bytes(ec_point[1:order_bytes+1])
+        y = Integer.from_bytes(ec_point[order_bytes+1:])
+    # Compressed point
+    elif point_type in (0x02, 0x3):
+        if len(ec_point) != (1 + order_bytes):
+            raise ValueError("Incorrect EC point length")
+        x = Integer.from_bytes(ec_point[1:])
+        y = (x**3 - x*3 + _curve.b).sqrt(_curve.p)    #  Short Weierstrass
+        if point_type == 0x02 and y.is_odd():
+            y = _curve.p - y
+        if point_type == 0x03 and y.is_even():
+            y = _curve.p - y
+    else:
+        raise ValueError("Incorrect EC point encoding")
 
-    point_x = Integer.from_bytes(publickey[1:order_bytes+1])
-    point_y = Integer.from_bytes(publickey[order_bytes+1:])
-    return construct(curve="P-256", point_x=point_x, point_y=point_y)
+    return construct(curve="P-256", point_x=x, point_y=y)
 
 
 def _import_subjectPublicKeyInfo(encoded, *kwargs):
-    oid, encoded_key, params = _expand_subject_public_key_info(encoded)
+    """Convert a subjectPublicKeyInfo into an EccKey object"""
+
+    # See RFC5480
+
+    # Parse the generic subjectPublicKeyInfo structure
+    oid, ec_point, params = _expand_subject_public_key_info(encoded)
+
+    # ec_point must be an encoded OCTET STRING
+    # params is encoded ECParameters
 
     # We accept id-ecPublicKey, id-ecDH, id-ecMQV without making any
     # distiction for now.
-    unrestricted_oid = "1.2.840.10045.2.1"
+    unrestricted_oid = "1.2.840.10045.2.1"  # Restrictions can be captured
+                                            # in the key usage certificate
+                                            # extension
     ecdh_oid = "1.3.132.1.12"
     ecmqv_oid = "1.3.132.1.13"
 
-    if oid not in (unrestricted_oid, ecdh_oid, ecmqv_oid) or not params:
+    if oid not in (unrestricted_oid, ecdh_oid, ecmqv_oid):
         raise UnsupportedEccFeature("Unsupported ECC purpose (OID: %s)" % oid)
 
+    # Parameters are mandatory for all three types
+    if not params:
+        raise ValueError("Missing ECC parameters")
+
     # ECParameters ::= CHOICE {
     #   namedCurve         OBJECT IDENTIFIER
     #   -- implicitCurve   NULL
     #   -- specifiedCurve  SpecifiedECDomain
     # }
-    curve_name = DerObjectId().decode(params).value
+    #
+    # implicitCurve and specifiedCurve are not supported (as per RFC)
+    curve_oid = DerObjectId().decode(params).value
 
-    return _import_public_der(curve_name, encoded_key)
+    return _import_public_der(curve_oid, ec_point)
 
 
 def _import_private_der(encoded, passphrase, curve_name=None):
@@ -713,20 +789,33 @@ def _import_x509_cert(encoded, *kwargs):
 
 def _import_der(encoded, passphrase):
 
-    decodings = (
-        _import_subjectPublicKeyInfo,
-        _import_x509_cert,
-        _import_private_der,
-        _import_pkcs8,
-        )
-
-    for decoding in decodings:
-        try:
-            return decoding(encoded, passphrase)
-        except UnsupportedEccFeature, uef:
-            raise uef
-        except (ValueError, TypeError, IndexError):
-            pass
+    try:
+        return _import_subjectPublicKeyInfo(encoded, passphrase)
+    except UnsupportedEccFeature, err:
+        raise err
+    except (ValueError, TypeError, IndexError):
+        pass
+    
+    try:
+        return _import_x509_cert(encoded, passphrase)
+    except UnsupportedEccFeature, err:
+        raise err
+    except (ValueError, TypeError, IndexError):
+        pass
+    
+    try:
+        return _import_private_der(encoded, passphrase)
+    except UnsupportedEccFeature, err:
+        raise err
+    except (ValueError, TypeError, IndexError):
+        pass
+    
+    try:
+        return _import_pkcs8(encoded, passphrase)
+    except UnsupportedEccFeature, err:
+        raise err
+    except (ValueError, TypeError, IndexError):
+        pass
 
     raise ValueError("Not an ECC DER key")
 
